; ---------------------------------------------------------------------
; Kanalschalter fr VTX mit ATTiny13	   20.03.2018 (c) Jrg Reisener
; letzte nderung:			   22.03.2018
; ---------------------------------------------------------------------
;
; RC-Schalter zur Kanalumschaltung am VTX-Modul SM130T
; reagiert auf PPM Signal, Kanal 8
;
; Servoimpulslnge <1,3ms - CS1=0, CS2=0 - 1240MHz
; Servoimpulslnge <1,7ms - CS1=1, CS2=0 - 1280MHz
; Servoimpulslnge >1,7ms - CS1=0, CS2=1 - 1320MHz
; kein PPM Signal	  - CS1=1, CS2=1 - 1360MHz
; 1=tristate, 0=low
; 
; Fusebits wie Lieferzustand
; -U lfuse:w:0x6a:m -U hfuse:w:0xff:m
; CPU-Takt 600kHz
;
; ---------------------------------------------------------------------
; ----------------------
; Belegung der I/O-Ports
; ----------------------
;
; Pin1/RESET:		unbeschaltet
; Pin2/PortB3:		Ausgang CS1
; Pin3/PortB4:		Ausgang CS2
; Pin4:			GND
; Pin5/PortB0:		unbeschaltet
; Pin6/PortB1:		Eingang PPM-Signal (INT0)
; Pin7/PortB2/ADC1:	unbeschaltet
; Pin8:			Vcc
;
; -----------------------------------
; Konstanten und Registerdefinitionen
; -----------------------------------
;
.nolist
.include "tn13def.inc"
.list
;
.equ	kanal=	8	;Auswahl ppm Kanal
.equ	rcout1=	3	;Schaltausgang CS1
.equ	rcout2=	4	;Schaltausgang CS2
.equ	ms07=	53	;Timerwert fr 0,7ms
.equ	ms13=	98	;Timerwert fr 1,3ms
.equ	ms17=	128	;Timerwert fr 1,7ms
.equ	ms25=	188	;Timerwert fr 2,5ms

;
.def	insreg=	r1	;Zwischenspeicher fr SREG whrend Interrupt
.def	impuls=	r20	;Speicher fr Servoimpulslnge
.def	chan=	r21	;Kanalzhler
.def	itemp=	r25	;Zwischenspeicher whrend Interruptbearbeitung
;
; ---------------
; Programmbereich
; ---------------
;
.cseg
.org 0
	rjmp	start			;Programmstart
	rjmp	exint0			;ext. Interrupt PPM-Signal
	reti
	rjmp	timovf			;Timer Overflow Interrupt
;
.db	"VTX-PPM-Schalter    "
.db	"mit ATTiny13        "
.db	"03/2018             "
.db	"(c) Joerg Reisener  "
;
; Initialisierung
;
start:	ldi	r16,ramend		;Stackpointer auf
	out	spl,r16			;RAM-Ende setzen
	ldi	r16,0b00000000
	out	ddrb,r16		;alles Eingnge, tristate
	ldi	r16,0b00000111		;Pullup B0-B2 ein, Ausgnge low
	out	portb,r16
;
	ldi	r16,(1<<clkpce)		;Clock Prescaler nderung erlauben
	out	clkpr,r16
	ldi	r16,(1<<clkps2)		;Clock Prescaler auf 16
	out	clkpr,r16
;
	ldi	r16,(1<<isc01)		;INT0 fallende Flanke
	out	mcucr,r16
	ldi	r16,(1<<int0)		;erlaube Interrupt
	out	gimsk,r16
;
	ldi	r16,(1<<cs01)		;Timer Vorteiler 8
	out	tccr0b,r16
	ldi	r16,(1<<toie0)		;Timer Overflow Int. enable
	out	timsk0,r16
;
	rcall	w330ms			;warten
	clr	impuls			;Impulsspeicher lschen
	ldi	chan,17			;Kanalzhler Initialwert
	clr	r16
	out	tcnt0,r16		;Timer auf 0 setzen
	sei				;Interrupt erlauben
;
; Hauptschleife
;
main:
	cpi	impuls,ms07		;Impuls < 0,7ms?
	brcs	ma0200			;ja -> Impulsfehler
	cpi	impuls,ms25		;Impuls < 2,5ms?
	brcc	ma0200			;nein -> Impulsfehler
	cpi	impuls,ms13		;Impuls < 1,3ms?
	brcs	ma0100			;ja -> 1240MHz
	cpi	impuls,ms17		;Impuls < 1,7ms?
	brcs	ma0110			;ja -> 1280MHz
ma0010:					;1320MHz
	sbi	ddrb,rcout1		;CS1 low
	cbi	ddrb,rcout2		;CS2 tristate
	rjmp	ma0300			;zum Ende
ma0100:					;1240MHz
	sbi	ddrb,rcout1		;CS1 auf low
	sbi	ddrb,rcout2		;CS2 auf low
	rjmp	ma0300			;zum Ende
ma0110:					;1280MHz
	cbi	ddrb,rcout1		;CS1 tristate
	sbi	ddrb,rcout2		;CS2 auf low
	rjmp	ma0300			;zum Ende
ma0200:					;1360MHz
	cbi	ddrb,rcout1		;CS1 tristate
	cbi	ddrb,rcout2		;CS2 tristate
ma0300:	rjmp	main			;Ende Hauptschleife
;
; ----------------------
; Unterprogramm-Routinen
; ----------------------
;
; Warteschleife 330ms
;
w330ms:	push	r16
	push	r17
	clr	r16
	clr	r17
wloop:	dec	r16			;innere Schleife
	brne	wloop			;luft 1280ys
	dec	r17
	brne	wloop			;uere Schleife
	pop	r17
	pop	r16
	ret
;
; ------------------
; Interrupt-Routinen
; ------------------
;
; Externer Interrupt INT0
;
exint0:	in	insreg,sreg		;SREG sichern
	cpi	chan,kanal		;ausgewhlter Kanal erreicht?
	brne	ex0010			;nein: berspringen
	in	impuls,tcnt0		;Timerwert speichern
ex0010:	inc	chan			;Kanalzhler erhhen
	clr	itemp
	out	tcnt0,itemp		;Timer0 zurcksetzen
	out	sreg,insreg		;SREG wiederherstellen
	reti
;
; Timer Overflow Interrupt
;
timovf:	in	insreg,sreg		;SREG sichern
	clr	chan			;Kanalzhler zurcksetzen
	out	sreg,insreg		;SREG wiederherstellen
	reti
